<?php 
/**
 * Plugin Name: Goal Conversion Export (Matomo Plugin)
 * Plugin URI: http://plugins.matomo.org/GoalConversionExport
 * Description: Provide an exporter for Google Ads and Facebook Ads
 * Author: Benjamin Tourman
 * Author URI: https://github.com/btourman/goal-conversion-export
 * Version: 0.1.0
 */
?><?php
/**
 * Matomo - free/libre analytics platform
 *
 * @link https://matomo.org
 * @license http://www.gnu.org/licenses/gpl-3.0.html GPL v3 or later
 */

namespace Piwik\Plugins\GoalConversionExport;

use Piwik\Common;
use Piwik\Plugins\GoalConversionExport\Dao\GoalConversionModel;
use Piwik\Plugins\GoalConversionExport\Dao\LogClickIdModel;

 
if (defined( 'ABSPATH')
&& function_exists('add_action')) {
    $path = '/matomo/app/core/Plugin.php';
    if (defined('WP_PLUGIN_DIR') && WP_PLUGIN_DIR && file_exists(WP_PLUGIN_DIR . $path)) {
        require_once WP_PLUGIN_DIR . $path;
    } elseif (defined('WPMU_PLUGIN_DIR') && WPMU_PLUGIN_DIR && file_exists(WPMU_PLUGIN_DIR . $path)) {
        require_once WPMU_PLUGIN_DIR . $path;
    } else {
        return;
    }
    add_action('plugins_loaded', function () {
        if (function_exists('matomo_add_plugin')) {
            matomo_add_plugin(__DIR__, __FILE__, true);
        }
    });
}

class GoalConversionExport extends \Piwik\Plugin
{
    public function registerEvents()
    {
        return [
            'CronArchive.getArchivingAPIMethodForPlugin' => 'getArchivingAPIMethodForPlugin',
            'Db.getTablesInstalled' => 'getTablesInstalled',
        ];
    }

    public function getTablesInstalled(&$allTablesInstalled)
    {
        $allTablesInstalled[] = Common::prefixTable('goal_conversion_export');
        $allTablesInstalled[] = Common::prefixTable('goal_conversion_log_click_id');
    }

    public function install()
    {
        $goalConversionModel = new GoalConversionModel();
        $goalConversionModel->install();
        $logClickIdModel = new LogClickIdModel();
        $logClickIdModel->install();
    }

    public function uninstall()
    {
        $goalConversionModel = new GoalConversionModel();
        $goalConversionModel->uninstall();
        $logClickIdModel = new LogClickIdModel();
        $logClickIdModel->uninstall();
    }

    public function isTrackerPlugin()
    {
        return true;
    }

}
